/* level_set.c (Combinatorial Pyramids project)
 *  Copyright (C) 2003 Luc Brun
 *  email: luc.brun@univ-reims.fr
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include "level_set.h"

#include <assert.h>
#include "objet.h"
#include "tab_int.h"

#define ABS(a) ((a)<0?(-(a)):(a))
#define MAX(a,b) ((a)<(b)?((b)):(a))

#define Max_level      SELF(max_level)
#define Nb_edge        SELF(nb_edge)
#define Levels         SELF(levels)
#define Darts          SELF(darts)
#define Dart_index     SELF(dart_index)
#define Current        SELF(current)
#define Current_level  SELF(current_level)

static void load_update_levels_priv(dart,dart,
				    int, int,
				    tab_int ,
				    tab_int);
static void load_get_levels_priv(image,int *,int *);


struct level_set
{
  int max_level;
  int nb_edge;
  tab_int levels;
  int* dart_index;
  dart* darts;
  int current;
  int current_level;
};
/* Definition of level : \forall i\in\{1,\dots,max\_level\}, \;
   \forall j \geq level (i),   level(Darts[j])<= i  */

extern level_set new_level_set(int nb_edge)
{
	int i;

	INSTANCIER(level_set);

	Max_level=1;
	Nb_edge=nb_edge;
	Darts=(dart*) malloc(Nb_edge*sizeof(dart));
	Dart_index=(int*)malloc(Nb_edge*sizeof(int));
	Dart_index-=1;
	Current=0;
	Levels=FAIRE_tab_int();

	for(i=0;i<Nb_edge;i++)
	{
		Dart_index[i+1]=i;
		Darts[i]=i+1;
	}  
	tab_int_put_value(Levels,0,Nb_edge);
	tab_int_put_value(Levels,1,0);

	return self;
}

extern void destroy_level_set(level_set self)
{
	assert(self);

	free(Darts);
	free(Dart_index+1);
	DEFAIRE_tab_int(Levels);

	free(self);  
}

extern level_set level_set_load(image img,tab_int levels)
{
	int level1,level2;
	int dimx,dimy;
	dart right,down;
	int i,j;
	int index_level;
	int level,index;
	tab_int currents;
	tab_int nb_dart_per_level;
	
	INSTANCIER(level_set);

	Max_level=0;
	dimx=image_give_largeur(img)-1;
	dimy=image_give_hauteur(img)-1;

	Nb_edge=dimx*(dimy+1)+(dimx+1)*dimy;
	Darts=(dart*) malloc(Nb_edge*sizeof(dart));
	Dart_index=(int*)malloc(Nb_edge*sizeof(int));
	Dart_index-=1;
	Current=0;
	Levels=FAIRE_tab_int();

	currents=FAIRE_tab_int();
	nb_dart_per_level=FAIRE_tab_int();

	right=1;
	down=dimx*(dimy+1)+1;
	image_debut(img);
	for(i=0;i<dimy;i++)
	{
		for(j=0;j<dimx;j++)
		{
			load_get_levels_priv(img,&level1,&level2);
			load_update_levels_priv(right,down,
			level1,level2,
			levels,nb_dart_per_level);
			Max_level=MAX(Max_level,level1);
			Max_level=MAX(Max_level,level2);
			image_pixel_suivant(img);
			right++;
			down++;
		}
		load_get_levels_priv(img,NULL,&level2);
		load_update_levels_priv(0,down,
		                        0,level2,
		                        levels,nb_dart_per_level);
		Max_level=MAX(Max_level,level2);
		image_pixel_suivant(img);
		down++;
	}
	for(j=0;j<dimx;j++)
	{
		load_get_levels_priv(img,&level1,NULL);
		load_update_levels_priv(right,0,
		                        level1,0,
		                        levels,nb_dart_per_level);
		Max_level=MAX(Max_level,level1);
		image_pixel_suivant(img);
		right++;
	}
	Current=0;
	index_level=Nb_edge;
	for(i=0;i<Max_level;i++)
	{
		index_level-=tab_int_give_value(nb_dart_per_level,i);
		tab_int_put_value(Levels,i,index_level);
		tab_int_put_value(currents,i,index_level);
	}
	tab_int_put_value(currents,Max_level,0);

	for(i=1;i<=Nb_edge;i++)
	{
		level=tab_int_give_value(levels,i);
		index=tab_int_give_value(currents,level);
		Darts[index]=i;
		Dart_index[i]=index;
		index++;
		tab_int_put_value(currents,level,index);
	}
	/*    level_set_debug(self);*/

	DEFAIRE_tab_int(currents);
	DEFAIRE_tab_int(nb_dart_per_level);

	return self;
}

extern int level_set_give_max_level(level_set self)
{
	return Max_level;
}
extern void level_set_new_level(level_set self)
{
	Max_level++;

	tab_int_put_value(Levels,Max_level,0);
	tab_int_put_value(Levels,Max_level-1,
	                  tab_int_give_value(Levels,Max_level-2));
}
extern void level_set_first(level_set self,int level)
{
	Current=tab_int_give_value(Levels,level);
	Current_level=level;
}
extern dart level_set_current(level_set self)
{
	return Darts[Current];
}
extern bouleen level_set_next(level_set self)
{
	int lower_bound=tab_int_give_value(Levels,Current_level-1);

	if(Current+1>= lower_bound)
		return faux;
	Current++;
	return vrai;
}
extern void level_set_suppress_edge(level_set self,dart d)
{
	int level=tab_int_give_value(Levels,Max_level-1)-1;
	int ind,edge,b;

	assert(self&&d);

	edge=ABS(d);
	ind=Dart_index[edge];
	if(ind == level)				/* a voir. */
	{
		if(Current >= level)
			Current=level-1;
		tab_int_put_value(Levels,Max_level-1,level);
		return;
	}
	b=Darts[level];
	Darts[level]=edge;
	Darts[ind]=b;
	Dart_index[edge]=level;
	Dart_index[b]=ind;

	tab_int_put_value(Levels,Max_level-1,level);
}

extern tab_int level_set_give_all_levels(level_set self)
{
	tab_int levels=FAIRE_tab_int();
	int level;
	int i,min_ind,max_ind;

	for(level=1;level<=Max_level;level++)
	{
		min_ind=tab_int_give_value(Levels,level);
		max_ind=tab_int_give_value(Levels,level-1);
		for(i=min_ind;i<max_ind;i++)
			tab_int_put_value(levels,Darts[i],level);
	}
	return levels;
}

static void load_get_levels_priv(image img,
                                 int * level1,
                                 int * level2)
{
	int* pixel;

	pixel=image_lire_pixel(img);

	if(level1!=NULL)
		*level1=pixel[0]|((pixel[1]&0x0f)<<8);
	if(level2 != NULL)
		*level2=((pixel[1]&0xf0)>>4)|((pixel[2]&0xff)<<4);
}

static void load_update_levels_priv(dart b1, dart b2,
                                    int level1, int level2,
                                    tab_int levels,
                                    tab_int nb_dart_per_level)
{
	int count;

	if(b1 != 0)
	{
		tab_int_put_value(levels,b1,level1);
		count=tab_int_give_value(nb_dart_per_level,level1);
		count++;
		tab_int_put_value(nb_dart_per_level,level1,count);
	}
	if(b2!= 0)
	{
		tab_int_put_value(levels,b2,level2);
		count=tab_int_give_value(nb_dart_per_level,level2);
		count++;
		tab_int_put_value(nb_dart_per_level,level2,count);
	}
}

extern void level_set_debug(level_set self)
{
	int i;
	fprintf(stderr,"Maxlevel : %d\n",Max_level);
	for(i=0;i<=Max_level;i++)
		fprintf(stderr,"Index[%d]=%d\n",i,tab_int_give_value(Levels,i));

	for(i=0;i<Nb_edge;i++)
		fprintf(stderr,"Darts[%d]=%d\n",i,Darts[i]);
}
