/* main.c (Combinatorial Pyramids project)
 *  Copyright (C) 2003 Luc Brun
 *  email: luc.brun@univ-reims.fr
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include "list_gen.h"
#include "tab_mark_dart.h"
#include "pyramid.h"
#include "image.h"
#include "pyramid_value.h"
#include "pyramid_draw_ppm.h"
#include "quantize.h"
#include "quantification.h"
#include "seq_segmentation.h"
#include "para_segmentation.h"
#include "para_nettoyage.h"

void save_image(pyramid P,int width,int height)
{
  static int iter=1;
  int yellow[3]={255,255,0};
  int red[3]={255,0,0};
  image result;
  char res_name[80];

  pyramid_draw_ppm_init(width,height,yellow,red);
  pyramid_draw_internal_boundaries(P,
                                   pyr_draw_both,
				   NULL_PFNEW_SEGMENT,
                                   pyramid_draw_ppm_draw_point);

  result=pyramid_draw_ppm_give_image();
  sprintf(res_name,"im%d.ppm",iter);
  printf("im%d.ppm\n",iter);
  image_sauvegarder(result,res_name);
  iter++;
}

int main(int argc,char* argv[])
{
  int width, height;
  pyramid P;
  image img=FAIRE_image();
  image res=FAIRE_image();
  int k=atoi(argv[1]);
  quantize quant;
  vertex_data v;
  dart_data  b;
  parallel_seg seg_par;
  sequential_seg seg_seq;



  // Loading of the image
  image_charger(img,argv[2]);
  width=image_give_largeur(img);
  height=image_give_hauteur(img);


  // Quantification
  quant=do_quantize(k,img,1);
  quantification(img, res, quant);


  // Initialisation of the data of the pyramid from the image values
  v.data=(any_ptr)new_pyramid_value(img,quant);
  v.init=init_node;
  v.merge=merge_value;
  b.data=(any_ptr)new_border_value(img);
  b.init=init_edge;
  b.merge=merge_border;


  // Initialisation of the pyramid reduction functions
  init_seq_segmentation(v.data, b.data);
  init_para_segmentation(v.data, b.data);
  init_para_nettoyage(v.data, b.data);


  // Creation of the pyramid
  P=new_pyramid(width,height,&v,&b);

printf("avant < pyramid_sequential_reduce >\n");
  seg_seq.equiv=equiv_node;
  seg_seq.merge=NULL_PFMERGE;
  seg_seq.merge_dart=NULL_PFMERGE_DART;
  pyramid_sequential_reduce(P,&seg_seq);

printf("avant < pyramid_parallel_reduce >\n");
  para_segmentation_put_seuil_lambda_min(600);
  para_segmentation_put_seuil_lambda_max(600);
  para_segmentation_put_progression_lambda(-0.025);
  para_segmentation_put_gradient_percent_value(0.9);
  seg_par.equiv=para_lambda;
  seg_par.merge=NULL_PFMERGE;
  seg_par.value=para_value;
  seg_par.interest=para_interest;
  seg_par.merge_dart=NULL_PFMERGE_DART;
  pyramid_parallel_reduce(P,&seg_par);

printf("avant Nettoyage\n");
  para_nettoyage_put_seuil_size(50);
  seg_par.equiv=para_nettoyage_lambda;
  seg_par.value=para_nettoyage_value;
  seg_par.merge_dart=NULL_PFMERGE_DART;
  pyramid_parallel_reduce(P,&seg_par);
  save_image(P,width,height);


  destroy_pyramid(P,free);
  if (img!=NULL_IMAGE)
    DEFAIRE_image(img);
  if (res!=NULL_IMAGE)
    DEFAIRE_image(res);

  return 0;
}
