/* pyramid_value.c (Combinatorial Pyramids project)
 *  Copyright (C) 2003 Luc Brun
 *  email: luc.brun@univ-reims.fr
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include "pyramid_value.h"
#include <assert.h>

static bouleen rm_noise=faux;

#define VM1	SELF(M1)
#define VM2	SELF(M2)
#define VMerged SELF(merged)
#define VCardinal SELF(cardinal)
#define VLabel SELF(label)
#define VPerimetre SELF(perimetre)

struct vertex_value
{
  double M1[3];
  double M2[3];
  bouleen merged;
  int perimetre;
  int cardinal;
  int label;
};

#define Value_vertice SELF(value_vertice)
#define Img SELF(img)
#define Img_dim SELF(img_dim)
#define Quant SELF(quant)

struct pyramid_value
{
  tab_gen value_vertice;
  image img;
  int img_dim;
  quantize quant;
};

extern vertex_value new_vertex_value(int *pix,quantize quant,int img_dim)
{
  int i;
  INSTANCIER(vertex_value);
  for(i=0;i<img_dim;i++)
    {
      VM1[i]=pix[i];
      VM2[i]=pix[i]*pix[i];
    }
  VCardinal=1;
  VPerimetre=4;
  VMerged=faux;
  VLabel=quantize_give_color_index(quant,pix);

  return self;
 }
extern any_ptr init_node(any_ptr data,pyramid pyr,vertex v)
{
  pyramid_value self=(pyramid_value)data;
  int width=image_give_largeur(Img);
  vertex_value value=0;
  int* pix;
  Point P;

  COORDY(P)  = (v-1)/width;
  COORDX(P)  = (v-1)%width;

  image_move_to(Img,&P);
  pix=image_lire_pixel(Img);
  value=new_vertex_value(pix,Quant,Img_dim);
  tab_gen_insert_object(Value_vertice,value,v);
  
  return value;
}
extern void vertex_value_reset(vertex_value self,double*M1,double*M2,int C,int L,int img_dim,int perim)
{
  int i;
  for(i=0;i<img_dim;i++)
    {
      VM1[i]=M1[i];
      VM2[i]=M2[i];
    }
  VCardinal=C;
  VPerimetre=perim;
  VMerged=faux;
  VLabel=L;
}

extern void vertex_value_give_color(vertex_value self,double*color)
{
  int k;
  assert(self);
  for(k=0;k<3;k++)
    color[k]=VM1[k];
}
extern void vertex_value_give_squared_color(vertex_value self,double*color)
{
  int k;
  assert(self);
  for(k=0;k<3;k++)
    color[k]=VM2[k];
}
extern int vertex_value_give_cardinal(vertex_value self)
{
  assert(self);
  return VCardinal;
}

extern void vertex_value_give_mean(vertex_value self,double*color)
{
  int k;
  assert(self);
  for(k=0;k<3;k++)
    color[k]=VM1[k]/VCardinal;
}
extern double vertex_value_give_variance(vertex_value self)
{
  double res=0, tmp;
  int k;
  assert(self);
  for(k=0;k<3;k++)
  {
      tmp=VM1[k]/VCardinal;
      res+=VM2[k]/VCardinal+tmp*tmp;
  }
  return tmp;
}
extern double vertex_value_give_se(vertex_value self)
{
  double res=0;
  int k, C;
  assert(self);
  C =VCardinal;
  for(k=0;k<3;k++)
  	res+=VM2[k]-VM1[k]*VM1[k]/C;
  return res;
}
extern int vertex_value_give_label(vertex_value self)
{
  assert(self);
  return VLabel;
}
extern int vertex_value_give_perimetre(vertex_value self)
{
  assert(self);
  return VPerimetre;
}
extern void vertex_value_add_perimetre(vertex_value self, int v)
{
  VPerimetre+=v;
}


/*-----------------------------------------------------------------------------*/

extern pyramid_value new_pyramid_value(image I,quantize quant)
{
  INSTANCIER(pyramid_value);
  Value_vertice=FAIRE_tab_gen();
  Img=I;
  Img_dim=image_give_dim(I);
  Quant=quant;
  return self;
}
extern void destroy_pyramid_value(pyramid_value self)
{
  int i=tab_gen_give_cardinal(Value_vertice);
  any_ptr obj;

  while(i--)
  {
    obj=tab_gen_give_object(Value_vertice,i);
    if(obj != NULL)
      free(obj);
  }
  DEFAIRE_tab_gen(Value_vertice);
  self=NULL;
}
extern void pyramid_value_release_after_init(pyramid_value self)
{
  if (Img!=NULL_IMAGE){
    DEFAIRE_image(Img);
    Img=NULL_IMAGE;
  }
  if (Quant!=NULL){
    destroy_quantize(Quant);
    Quant=NULL;
  }
}
extern any_ptr pyramid_value_give_vertex_value(pyramid_value self,
                                               vertex mu)
{
  assert(self);
  return tab_gen_give_object(Value_vertice,mu);
}
extern void pyramid_value_remove_object(pyramid_value self, vertex mu)
{
  tab_gen_remove_object(Value_vertice,mu);
}
extern void  init_pyramid_value(image I,quantize quant)
{
/*
  Img=I;
  Img_dim=image_give_dim(I);
  Quant=quant;
*/
}
extern void pyramid_value_reset(pyramid_value self, vertex mu, double*M1, double*M2, int C, int L, int perim)
{
  vertex_value v;
  assert(self);
  v=(vertex_value)tab_gen_give_object(Value_vertice,mu);
  vertex_value_reset(v, M1, M2, C, L, Img_dim,perim);
}
extern void pyramid_value_give_color(pyramid_value self,vertex mu,double*color)
{
  vertex_value v; 
  assert(self);
  v=(vertex_value)tab_gen_give_object(Value_vertice,mu);
  vertex_value_give_color(v,color);
}
extern int pyramid_value_give_cardinal(pyramid_value self,vertex mu)
{
  vertex_value v; 
  assert(self);
  v=(vertex_value)tab_gen_give_object(Value_vertice,mu);
  return vertex_value_give_cardinal(v);
}

extern void pyramid_value_give_mean(pyramid_value self,vertex mu,double*color)
{
  vertex_value v; 
  assert(self);
  v=(vertex_value)tab_gen_give_object(Value_vertice,mu);
  vertex_value_give_mean(v,color);
}

extern double pyramid_value_give_variance(pyramid_value self,vertex mu)
{
  vertex_value v; 
  assert(self);
  v=(vertex_value)tab_gen_give_object(Value_vertice,mu);
  return vertex_value_give_variance(v);
}
extern double pyramid_value_give_se(pyramid_value self,vertex mu)
{
  vertex_value v; 
  assert(self);
  v=(vertex_value)tab_gen_give_object(Value_vertice,mu);
  return vertex_value_give_se(v);
}
extern int pyramid_value_give_label(pyramid_value self,vertex mu)
{
  vertex_value v; 
  assert(self);
  v=(vertex_value)tab_gen_give_object(Value_vertice,mu);
  return vertex_value_give_label(v);  
}
extern int pyramid_value_give_perimetre(pyramid_value self,vertex mu)
{
  vertex_value v; 
  assert(self);
  v=(vertex_value)tab_gen_give_object(Value_vertice,mu);
  return vertex_value_give_perimetre(v);  
}
extern void pyramid_value_add_perimetre(pyramid_value self,vertex mu,int val)
{
  vertex_value v; 
  assert(self);
  v=(vertex_value)tab_gen_give_object(Value_vertice,mu);
  vertex_value_add_perimetre(v, val);
}
extern int pyramid_value_give_dim_img(pyramid_value self)
{
  return Img_dim;
}

extern void value_remove_noise()
{
  rm_noise=vrai;
}




extern any_ptr merge_value(any_ptr data,
			   pyramid P, 
                           int level,
                           vertex v1,vertex v2)
{
  int k;
  pyramid_value self=(pyramid_value)data;
  vertex_value value1=(vertex_value)tab_gen_give_object(Value_vertice,
  							v1);
  vertex_value value2=(vertex_value)tab_gen_give_object(Value_vertice,
  							v2);

  value1->cardinal+=value2->cardinal;
  
  vertex_value_add_perimetre(value1,vertex_value_give_perimetre(value2));
  
  for(k=0;k<Img_dim;k++)
    {
      value1->M1[k]+=value2->M1[k];
      value1->M2[k]+=value2->M2[k];
    }

  tab_gen_remove_object(Value_vertice,v2);
  free(value2);
  return value1;
}

















